@@ -17,7 +17,7 @@
 # along with the occi-py library.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-from occi.core import Entity
+from occi.core import Category, Entity, Mixin
 from occi.server import ServerBackend
 from occi.http import get_parser, get_renderer
 from occi.http.header import HttpHeaderError
@@ -91,7 +91,8 @@ class HandlerBase(object):
     def _request_init(self, request):
         """Parse request and initialize response renderer."""
         try:
-            parser = get_parser(request.content_type)
+            parser = get_parser(request.content_type,
+                    translator=self.translator)
             parser.parse(request.headers, request.body)
         except (ParserError, HttpHeaderError) as e:
             raise HttpRequestError(hrc.BAD_REQUEST(e))
@@ -166,6 +167,7 @@ class HandlerBase(object):
             print e
             raise HttpRequestError(hrc.SERVER_ERROR())
 
+
 class EntityHandler(HandlerBase):
     """HTTP handler for existing Entity instances."""
 
@@ -388,13 +390,51 @@ class CollectionHandler(HandlerBase):
     def _collection_action(self, request, path):
         return hrc.NOT_IMPLEMENTED()
 
+    def _update_mixin_collection(self, request, path, add=True):
+        # Parse request
+        parser, renderer = self._request_init(request)
+
+        # Lookup Mixin at the specified path
+        mixin = self.server.registry.lookup_location(path)
+        if not mixin:
+            raise HttpRequestError(hrc.NOT_FOUND('%s: no such collection' % path))
+        if not isinstance(mixin, Mixin):
+            raise HttpRequestError(hrc.BAD_REQUEST('%s: not a mixin location' % path))
+
+        # Get entities corresponding to the given locations
+        entities = []
+        for dao in parser.objects:
+            if not dao.location:
+                return hrc.BAD_REQUEST('resource instance location expected')
+            entity_id = self.translator.to_native(dao.location)
+            entity = self._get_entity(entity_id, user=request.user)
+            if add:
+                entity.occi_add_mixin(mixin)
+            else:
+                entity.occi_remove_mixin(mixin)
+            entities.append(entity)
+
+        # Save all updated entities using a single backend operation
+        try:
+            self._save_entities(entities, user=request.user)
+        except HttpRequestError as e:
+            return e.response
+
     def put(self, request, path):
-        """Add resource instance to Mixin collection"""
-        return hrc.NOT_IMPLEMENTED()
+        """Add resource instance(s) to Mixin collection"""
+        try:
+            self._update_mixin_collection(request, path, add=True)
+        except HttpRequestError as e:
+            return e.response
+        return hrc.ALL_OK()
 
     def delete(self, request, path):
-        """Remove resource instance from Mixin collection"""
-        return hrc.NOT_IMPLEMENTED()
+        """Remove resource instance(s) from Mixin collection"""
+        try:
+            self._update_mixin_collection(request, path, add=False)
+        except HttpRequestError as e:
+            return e.response
+        return hrc.ALL_OK()
 
 class DiscoveryHandler(HandlerBase):
     """HTTP handler for the OCCI discovery interface."""
@@ -420,9 +460,110 @@ class DiscoveryHandler(HandlerBase):
         return hrc.BAD_REQUEST()
 
     def put(self, request):
-        """Create a custem Mixin instance"""
-        return hrc.NOT_IMPLEMENTED()
+        """Create user-defined Mixin instance(s)"""
+        # Parse request and extract Mixin defs
+        try:
+            parser, renderer = self._request_init(request)
+        except HttpRequestError as e:
+            return e.response
+
+        # Any Mixin defs supplied?
+        if not parser.objects:
+            return hrc.BAD_REQUEST('No Mixin definition(s) supplied')
+
+        # Extract Mixin definitions
+        mixins = []
+        try:
+            for category in parser.objects[0].categories:
+                # Required parameters
+                if not hasattr(category, 'related'):
+                    return hrc.BAD_REQUEST('"%s": not a Mixin' % category)
+                if not hasattr(category, 'location') or not category.location:
+                    return hrc.BAD_REQUEST('"%s": location must be specified' % category)
+
+                # Mixin must not exist
+                try:
+                    self.server.registry.lookup_id(category)
+                except Category.DoesNotExist:
+                    pass
+                else:
+                    return hrc.BAD_REQUEST('%s: Category already exist' % category)
+
+                # Mixin location must not be used
+                location = self.translator.to_native(category.location) 
+                if self.server.registry.lookup_location(location):
+                    return hrc.BAD_REQUEST('%s: conflicting location path' % location)
+
+                # Create Mixin instance
+                mixin = Mixin(category.term, category.scheme,
+                        userdefined=True,
+                        related=category.related, location=location)
+                mixins.append(mixin)
+        except Category.Invalid as e:
+            return hrc.BAD_REQUEST(e)
+
+        # Store Mixins in backend and save to Category registry
+        try:
+            for mixin in mixins:
+                mixin = self.server.backend.add_user_mixin(mixin, user=request.user)
+                try:
+                    # FIXME: locking needed to avoid race condition
+                    self.server.registry.register(mixin)
+                except Category.Invalid as e:
+                    self.server.backend.remove_user_mixin(mixin, user=request.user)
+                    return hrc.BAD_REQUEST(e)
+        except ServerBackend.InvalidOperation as e:
+            return hrc.BAD_REQUEST(e)
+        except ServerBackend.ServerBackendError as e:
+            print e
+            return hrc.SERVER_ERROR()
+        except NotImplementedError as e:
+            return hrc.NOT_IMPLEMENTED(e)
+
+        return hrc.ALL_OK()
 
     def delete(self, request):
-        """Remove a custem Mixin instance"""
-        return hrc.NOT_IMPLEMENTED()
+        """Remove user-defined Mixin instance(s)"""
+        # Parse request and extract Mixin defs
+        try:
+            parser, renderer = self._request_init(request)
+        except HttpRequestError as e:
+            return e.response
+
+        # Any Mixin defs supplied?
+        if not parser.objects:
+            return hrc.BAD_REQUEST('No Mixin definition(s) supplied')
+
+        # Extract Mixin definitions
+        mixins = []
+        try:
+            for category in parser.objects[0].categories:
+                # Find Mixin in registry
+                try:
+                    mixin = self.server.registry.lookup_id(category)
+                except Category.DoesNotExist:
+                    return hrc.BAD_REQUEST('%s: does not exist' % category)
+
+                if not hasattr(mixin, 'userdefined') or not mixin.userdefined:
+                    return hrc.BAD_REQUEST('%s: not a user-defined Mixin' % mixin)
+                mixins.append(mixin)
+        except Category.Invalid as e:
+            return hrc.BAD_REQUEST(e)
+
+        # Remove Mixins from backend and Category registry
+        try:
+            for mixin in mixins:
+                self.server.backend.remove_user_mixin(mixin, user=request.user)
+                try:
+                    self.server.registry.unregister(mixin)
+                except Category.Invalid as e:
+                    return hrc.BAD_REQUEST(e)
+        except ServerBackend.InvalidOperation as e:
+            return hrc.BAD_REQUEST(e)
+        except ServerBackend.ServerBackendError as e:
+            print e
+            return hrc.SERVER_ERROR()
+        except NotImplementedError as e:
+            return hrc.NOT_IMPLEMENTED(e)
+
+        return hrc.ALL_OK()
